@@ -2,6 +2,8 @@
 
 """Base classes for HTTP resources."""
 
+from __future__ import absolute_import, division, print_function
+
 __metaclass__ = type
 
 __all__ = [
@@ -46,10 +48,9 @@ except ImportError:
      import sha
      sha_constructor = sha.new
 
-from zope.app.pagetemplate.engine import TrustedAppPT
-from zope import component
+import six
 from zope.component import (
-    adapts,
+    adapter,
     getAdapters,
     getAllUtilitiesRegisteredFor,
     getGlobalSiteManager,
@@ -58,22 +59,24 @@ from zope.component import (
     getUtility,
     queryMultiAdapter,
     )
-from zope.component.interfaces import ComponentLookupError
 from zope.event import notify
-from zope.publisher.http import init_status_codes, status_reasons
 from zope.interface import (
     alsoProvides,
     implementer,
-    implements,
     implementedBy,
     providedBy,
     Interface,
     )
 from zope.interface.common.sequence import IFiniteSequence
-from zope.interface.interfaces import IInterface
+from zope.interface.interfaces import (
+    ComponentLookupError,
+    IInterface,
+    )
 from zope.location.interfaces import ILocation
+from zope.pagetemplate.engine import TrustedAppPT
 from zope.pagetemplate.pagetemplatefile import PageTemplateFile
 from zope.proxy import isProxy
+from zope.publisher.http import init_status_codes, status_reasons
 from zope.publisher.interfaces import NotFound
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.schema import ValidationError, getFieldsInOrder
@@ -173,7 +176,7 @@ def _default_html_renderer(value):
     return cgi.escape(unicode(value))
 
 
-@component.adapter(Interface, IField, IWebServiceClientRequest)
+@adapter(Interface, IField, IWebServiceClientRequest)
 @implementer(IFieldHTMLRenderer)
 def render_field_to_html(object, field, request):
     """Turn a field's current value into an XHTML snippet.
@@ -236,10 +239,10 @@ class ResourceJSONEncoder(simplejson.encoder.JSONEncoderForHTML):
         return IJSONPublishable(obj).toDataForJSON(self.media_type)
 
 
+@implementer(IJSONPublishable)
+@adapter(BaseItem)
 class JSONItem:
     """JSONPublishable adapter for lazr.enum."""
-    implements(IJSONPublishable)
-    adapts(BaseItem)
 
     def __init__(self, context):
         self.context = context
@@ -249,9 +252,9 @@ class JSONItem:
         return str(self.context.title)
 
 
+@implementer(IHTTPResource)
 class RedirectResource:
     """A resource that redirects to another URL."""
-    implements(IHTTPResource)
 
     def __init__(self, url, request):
         self.url = url
@@ -263,9 +266,9 @@ class RedirectResource:
         self.request.response.setHeader("Location", url)
 
 
+@implementer(IHTTPResource)
 class HTTPResource:
     """See `IHTTPResource`."""
-    implements(IHTTPResource)
 
     # Some interesting media types.
     WADL_TYPE = 'application/vnd.sun.wadl+xml'
@@ -420,15 +423,8 @@ class HTTPResource:
         if etag is not None:
             return etag
 
-        if media_type in [self.WADL_TYPE, self.DEPRECATED_WADL_TYPE]:
-            # The WADL representation of a resource only changes when
-            # the software itself changes. Thus, we don't need any
-            # special information for its ETag core.
-            etag_cores = ['']
-        else:
-            # For other media types, we calculate the ETag core by
-            # delegating to the subclass.
-            etag_cores = self._getETagCores(cache)
+        # We calculate the ETag core by delegating to the subclass.
+        etag_cores = self._getETagCores(cache)
 
         if etag_cores is None:
             return None
@@ -484,7 +480,6 @@ class HTTPResource:
         would rather have WADL, XHTML, or JSON.
         """
         content_types = self.getPreferredContentTypes()
-        preferences = []
         winner = None
         for media_type in self.SUPPORTED_CONTENT_TYPES:
             try:
@@ -933,7 +928,7 @@ class ReadWriteResource(HTTPResource):
                     allow_string += " DELETE"
                 self.request.response.setStatus(405)
                 self.request.response.setHeader("Allow", allow_string)
-        except Exception, e:
+        except Exception as e:
             exception_info = sys.exc_info()
             view = getMultiAdapter((e, self.request), name="index.html")
             try:
@@ -944,11 +939,12 @@ class ReadWriteResource(HTTPResource):
                 # web application rather than a web service. We'll
                 # re-raise the exception, let the publisher look up
                 # the view, and hopefully handle it better.  Note the careful
-                # reraising that ensures the original trackabck is preserved.
-                raise exception_info[0], exception_info[1], exception_info[2]
+                # reraising that ensures the original traceback is preserved.
+                six.reraise(
+                    exception_info[0], exception_info[1], exception_info[2])
 
             self.request.response.setStatus(ws_view.status)
-            if ws_view.status / 100 == 4:
+            if ws_view.status // 100 == 4:
                 # This exception corresponds to a client-side error
                 result = ws_view()
             else:
@@ -1095,7 +1091,7 @@ class EntryManipulatingResource(ReadWriteResource):
 
             try:
                 value = marshaller.marshall_from_json_data(original_value)
-            except (ValueError, ValidationError), e:
+            except (ValueError, ValidationError) as e:
                 errors.append(u"%s: %s" % (repr_name, e))
                 continue
 
@@ -1156,7 +1152,7 @@ class EntryManipulatingResource(ReadWriteResource):
                 try:
                     # Do any field-specific validation.
                     field.validate(value)
-                except ConstraintNotSatisfied, e:
+                except ConstraintNotSatisfied as e:
                     # Try to get a string error message out of
                     # the exception; otherwise use a generic message
                     # instead of whatever object the raise site
@@ -1168,10 +1164,10 @@ class EntryManipulatingResource(ReadWriteResource):
                         error = "Constraint not satisfied."
                     errors.append(u"%s: %s" % (repr_name, error))
                     continue
-                except RequiredMissing, e:
+                except RequiredMissing as e:
                     error = "Missing required value."
                     errors.append(u"%s: %s" % (repr_name, error))
-                except (ValueError, ValidationError), e:
+                except (ValueError, ValidationError) as e:
                     error = str(e)
                     if error == "":
                         error = "Validation error"
@@ -1228,6 +1224,7 @@ class EntryManipulatingResource(ReadWriteResource):
                 try:
                     new_value = getattr(self.entry, field.__name__)
                 except Unauthorized:
+                    new_value = old_value
                     if flag is not Unauthorized:
                         force_clear = True
                 if force_clear or new_value != old_value:
@@ -1263,9 +1260,9 @@ class EntryManipulatingResource(ReadWriteResource):
         pass
 
 
+@implementer(IEntryFieldResource, IJSONPublishable)
 class EntryFieldResource(FieldUnmarshallerMixin, EntryManipulatingResource):
     """An individual field of an entry."""
-    implements(IEntryFieldResource, IJSONPublishable)
 
     SUPPORTED_CONTENT_TYPES = [HTTPResource.JSON_TYPE,
                                HTTPResource.XHTML_TYPE]
@@ -1327,9 +1324,9 @@ class EntryFieldResource(FieldUnmarshallerMixin, EntryManipulatingResource):
                     % media_type)
 
 
+@implementer(IEntryField, ILocation)
 class EntryField:
     """A schema field bound by name to a particular entry."""
-    implements(IEntryField, ILocation)
 
     def __init__(self, entry, field, name):
         """Initialize with respect to a named field of an entry."""
@@ -1342,10 +1339,10 @@ class EntryField:
         self.__name__ = self.name
 
 
+@adapter(EntryField, IHTTPRequest)
+@implementer(IAbsoluteURL)
 class EntryFieldURL(AbsoluteURL):
     """An IAbsoluteURL adapter for EntryField objects."""
-    component.adapts(EntryField, IHTTPRequest)
-    implements(IAbsoluteURL)
 
     def __init__(self, entryfield, request):
         self.context = entryfield
@@ -1375,10 +1372,10 @@ def make_entry_etag_cores(field_details):
 
 
 
+@implementer(IEntryResource, IJSONPublishable)
 class EntryResource(CustomOperationResourceMixin,
                     FieldUnmarshallerMixin, EntryManipulatingResource):
     """An individual object, published to the web."""
-    implements(IEntryResource, IJSONPublishable)
 
     SUPPORTED_CONTENT_TYPES = [HTTPResource.WADL_TYPE,
                                HTTPResource.DEPRECATED_WADL_TYPE,
@@ -1644,7 +1641,7 @@ class EntryResource(CustomOperationResourceMixin,
                     tagged_values = field.getTaggedValue(
                         'lazr.restful.exported')
                     original_name = tagged_values['original_name']
-                except KeyError, e:
+                except KeyError:
                     # This field has no tagged values, or is missing
                     # the 'original_name' value. Its entry class was
                     # probably created by hand rather than by tagging
@@ -1766,11 +1763,11 @@ class EntryResource(CustomOperationResourceMixin,
         return None
 
 
+@implementer(ICollectionResource)
 class CollectionResource(BatchingResourceMixin,
                          CustomOperationResourceMixin,
                          ReadOnlyResource):
     """A resource that serves a list of entry resources."""
-    implements(ICollectionResource)
 
     def __init__(self, context, request):
         """Associate this resource with a specific object and request."""
@@ -1804,7 +1801,7 @@ class CollectionResource(BatchingResourceMixin,
                 self.request.response.setHeader('Content-Type', media_type)
                 return result
 
-            result = self.batch(entries)
+            result = self.batch(entries) + '}'
 
         self.request.response.setHeader('Content-type', self.JSON_TYPE)
         return result
@@ -1821,8 +1818,7 @@ class CollectionResource(BatchingResourceMixin,
             request = self.request
         result = super(CollectionResource, self).batch(entries, request)
         result += (
-            ', "resource_type_link" : ' + simplejson.dumps(self.type_url)
-            + '}')
+            ', "resource_type_link" : ' + simplejson.dumps(self.type_url))
         return result
 
     @property
@@ -1844,9 +1840,9 @@ class CollectionResource(BatchingResourceMixin,
             return adapter.collection_type_link
 
 
+@implementer(IServiceRootResource, IJSONPublishable)
 class ServiceRootResource(HTTPResource):
     """A resource that responds to GET by describing the service."""
-    implements(IServiceRootResource, IJSONPublishable)
 
     # A preparsed template file for WADL representations of the root.
     WADL_TEMPLATE = LazrPageTemplateFile('templates/wadl-root.pt')
@@ -2078,9 +2074,9 @@ class ServiceRootResource(HTTPResource):
             adapter.singular_type)
 
 
+@implementer(IEntry)
 class Entry:
     """An individual entry."""
-    implements(IEntry)
 
     def __init__(self, context, request):
         """Associate the entry with some database model object."""
@@ -2088,9 +2084,9 @@ class Entry:
         self.request = request
 
 
+@implementer(ICollection)
 class Collection:
     """A collection of entries."""
-    implements(ICollection)
 
     def __init__(self, context, request):
         """Associate the entry with some database model object."""
@@ -2098,10 +2094,10 @@ class Collection:
         self.request = request
 
 
+@implementer(IScopedCollection)
+@adapter(Interface, Interface, IWebServiceLayer)
 class ScopedCollection:
     """A collection associated with some parent object."""
-    implements(IScopedCollection)
-    adapts(Interface, Interface, IWebServiceLayer)
 
     def __init__(self, context, collection, request):
         """Initialize the scoped collection.
