@@ -2,6 +2,8 @@
 
 """Declaration helpers to define a web service."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'COLLECTION_TYPE',
@@ -17,7 +19,6 @@ __all__ = [
     'call_with',
     'collection_default_content',
     'error_status',
-    'exported',
     'export_as_webservice_collection',
     'export_as_webservice_entry',
     'export_destructor_operation',
@@ -25,6 +26,9 @@ __all__ = [
     'export_operation_as',
     'export_read_operation',
     'export_write_operation',
+    'exported',
+    'exported_as_webservice_collection',
+    'exported_as_webservice_entry',
     'generate_collection_adapter',
     'generate_entry_adapters',
     'generate_entry_interfaces',
@@ -48,7 +52,10 @@ from zope.interface import classImplements
 from zope.interface.advice import addClassAdvisor
 from zope.interface.interface import fromFunction, InterfaceClass, TAGGED_DATA
 from zope.interface.interfaces import IInterface, IMethod
-from zope.schema import getFields
+from zope.schema import (
+    getFields,
+    getFieldsInOrder,
+    )
 from zope.schema.interfaces import (
     IField,
     IObject,
@@ -90,7 +97,6 @@ from lazr.restful.utils import (
     make_identifier_safe,
     VersionedDict,
     VersionedObject,
-    is_total_size_link_active,
     )
 
 LAZR_WEBSERVICE_ACCESSORS = '%s.exported.accessors' % LAZR_WEBSERVICE_NS
@@ -155,6 +161,84 @@ def _get_interface_tags():
     return f_locals.setdefault(TAGGED_DATA, {})
 
 
+def _export_as_webservice_entry(interface,
+                                singular_name=None, plural_name=None,
+                                contributes_to=None, publish_web_link=True,
+                                as_of=None, versioned_annotations=None):
+    """Tag an interface as exported on the web service as an entry.
+
+    This is the core of export_as_webservice_entry and
+    exported_as_webservice_entry.
+    """
+    annotation_stack = VersionedDict()
+
+    if singular_name is None:
+        # By convention, interfaces are called IWord1[Word2...]. The default
+        # behavior assumes this convention and yields a singular name of
+        # "word1_word2".
+        my_singular_name = camelcase_to_underscore_separated(
+            interface.__name__[1:])
+    else:
+        my_singular_name = singular_name
+
+    # Turn the named arguments into a dictionary for the first exported
+    # version.
+    initial_version = dict(
+        type=ENTRY_TYPE, singular_name=my_singular_name,
+        plural_name=plural_name, contributes_to=contributes_to,
+        publish_web_link=publish_web_link, exported=True,
+        _as_of_was_used=(as_of is not None))
+
+    afterwards = versioned_annotations or []
+    for version, annotations in itertools.chain(
+            [(as_of, initial_version)], afterwards):
+        annotation_stack.push(version)
+
+        for key, value in annotations.items():
+            if annotations != initial_version:
+                # Make sure that the 'annotations' dict contains only
+                # recognized annotations.
+                if key not in ENTRY_ANNOTATION_KEYS:
+                    raise ValueError(
+                        'Unrecognized annotation for version "%s": '
+                        '"%s"' % (version, key))
+            annotation_stack[key] = value
+        # If this version provides a singular name but not a plural name,
+        # apply the default pluralization rule.
+        if (annotations.get('singular_name') is not None
+                and annotations.get('plural_name') is None):
+            annotation_stack['plural_name'] = (
+                annotations['singular_name'] + 's')
+
+    # When called from the @exported_as_webservice_entry decorator, this
+    # would overwrite any interface annotations made by method decorators;
+    # but there are currently no such annotations that are valid for methods
+    # of entry interfaces anyway.
+    interface.setTaggedValue(LAZR_WEBSERVICE_EXPORTED, annotation_stack)
+
+    # Set the name of the fields that didn't specify it using the
+    # 'export_as' parameter in exported(). This must be done here, because
+    # the field's __name__ attribute is only set when the interface is
+    # created.
+    for name, field in getFields(interface).items():
+        tag_stack = field.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)
+        if tag_stack is None or tag_stack.is_empty:
+            continue
+        if tag_stack['type'] != FIELD_TYPE:
+            continue
+        for version, tags in tag_stack.stack:
+            # Set 'as' for every version in which the field is published but
+            # no 'as' is specified. Also set 'original_name' for every
+            # version in which the field is published--this will help with
+            # performance optimizations around permission checks.
+            if tags.get('exported') != False:
+                tags['original_name'] = name
+                if tags.get('as') is None:
+                    tags['as'] = name
+
+    annotate_exported_methods(interface)
+
+
 def export_as_webservice_entry(singular_name=None, plural_name=None,
                                contributes_to=None, publish_web_link=True,
                                as_of=None, versioned_annotations=None):
@@ -164,6 +248,54 @@ def export_as_webservice_entry(singular_name=None, plural_name=None,
     actually not be exported on its own but instead will contribute its
     attributes/methods to other exported entries.
 
+    This function does not work with Python 3.  Rather than calling this
+    within the class definition, decorate the class with
+    @exported_as_webservice_entry instead.
+
+    :param singular_name: The human-readable singular name of the entry,
+        eg. "paintbrush".
+    :param plural_name: The human-readable plural name of the entry,
+        eg. "paintbrushes"
+    :param contributes_to: An optional list of exported interfaces to which
+        this interface contributes.
+    :param publish_web_link: This parameter is ignored unless there is
+        a correspondence between this web service's entries and the
+        pages on some website. If that is so, and if this parameter is
+        set to True, the representation of this entry will include a
+        web_link pointing to the corresponding page on the website. If
+        False, web_link will be omitted.
+    :param as_of: The first version of the web service to feature this entry.
+    :param versioned_annotations: A list of 2-tuples (version,
+        {params}), with more recent web service versions earlier in
+        the list and older versions later in the list.
+
+        A 'params' dictionary may contain the key 'exported', which
+        controls whether or not to publish the entry at all in the
+        given version. It may also contain the keys 'singular_name',
+        'plural_name', 'contributes_to', or 'publish_web_link', which
+        work just like the corresponding arguments to this method.
+    """
+    _check_called_from_interface_def('export_as_webservice_entry()')
+
+    def mark_entry(interface):
+        """Class advisor that tags the interface once it is created."""
+        _check_interface('export_as_webservice_entry()', interface)
+        _export_as_webservice_entry(
+            interface, singular_name=singular_name, plural_name=plural_name,
+            contributes_to=contributes_to, publish_web_link=publish_web_link,
+            as_of=as_of, versioned_annotations=versioned_annotations)
+        return interface
+
+    addClassAdvisor(mark_entry)
+
+
+class exported_as_webservice_entry:
+    """Mark the content interface as exported on the web service as an entry.
+
+    If contributes_to is a non-empty sequence of Interfaces, this entry will
+    actually not be exported on its own but instead will contribute its
+    attributes/methods to other exported entries.
+
     :param singular_name: The human-readable singular name of the entry,
         eg. "paintbrush".
     :param plural_name: The human-readable plural name of the entry,
@@ -187,77 +319,26 @@ def export_as_webservice_entry(singular_name=None, plural_name=None,
         'plural_name', 'contributes_to', or 'publish_web_link', which
         work just like the corresponding arguments to this method.
     """
-    _check_called_from_interface_def('export_as_webservice_entry()')
-    def mark_entry(interface):
-        """Class advisor that tags the interface once it is created."""
-        _check_interface('export_as_webservice_entry()', interface)
 
-        annotation_stack = VersionedDict()
+    def __init__(self, singular_name=None, plural_name=None,
+                 contributes_to=None, publish_web_link=True,
+                 as_of=None, versioned_annotations=None):
+        self.singular_name = singular_name
+        self.plural_name = plural_name
+        self.contributes_to = contributes_to
+        self.publish_web_link = publish_web_link
+        self.as_of = as_of
+        self.versioned_annotations = versioned_annotations
 
-        if singular_name is None:
-            # By convention, interfaces are called IWord1[Word2...]. The
-            # default behavior assumes this convention and yields a
-            # singular name of "word1_word2".
-            my_singular_name = camelcase_to_underscore_separated(
-                interface.__name__[1:])
-        else:
-            my_singular_name = singular_name
-
-        # Turn the named arguments into a dictionary for the first
-        # exported version.
-        initial_version = dict(
-            type=ENTRY_TYPE, singular_name=my_singular_name,
-            plural_name=plural_name, contributes_to=contributes_to,
-            publish_web_link=publish_web_link, exported=True,
-            _as_of_was_used=(not as_of is None))
-
-        afterwards = versioned_annotations or []
-        for version, annotations in itertools.chain(
-            [(as_of, initial_version)], afterwards):
-            annotation_stack.push(version)
-
-            for key, value in annotations.items():
-                if annotations != initial_version:
-                    # Make sure that the 'annotations' dict
-                    # contains only recognized annotations.
-                    if key not in ENTRY_ANNOTATION_KEYS:
-                        raise ValueError(
-                            'Unrecognized annotation for version "%s": '
-                            '"%s"' % (version, key))
-                annotation_stack[key] = value
-            # If this version provides a singular name but not a
-            # plural name, apply the default pluralization rule.
-            if (annotations.get('singular_name') is not None
-                and annotations.get('plural_name') is None):
-                annotation_stack['plural_name'] = (
-                    annotations['singular_name'] + 's')
-
-        interface.setTaggedValue(LAZR_WEBSERVICE_EXPORTED, annotation_stack)
-
-        # Set the name of the fields that didn't specify it using the
-        # 'export_as' parameter in exported(). This must be done here,
-        # because the field's __name__ attribute is only set when the
-        # interface is created.
-        for name, field in getFields(interface).items():
-            tag_stack = field.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)
-            if tag_stack is None or tag_stack.is_empty:
-                continue
-            if tag_stack['type'] != FIELD_TYPE:
-                continue
-            for version, tags in tag_stack.stack:
-                # Set 'as' for every version in which the field is
-                # published but no 'as' is specified. Also set
-                # 'original_name' for every version in which the field
-                # is published--this will help with performance
-                # optimizations around permission checks.
-                if tags.get('exported') != False:
-                    tags['original_name'] = name
-                    if tags.get('as') is None:
-                        tags['as'] = name
-
-        annotate_exported_methods(interface)
+    def __call__(self, interface):
+        _check_interface('exported_as_webservice_entry()', interface)
+        _export_as_webservice_entry(
+            interface,
+            singular_name=self.singular_name, plural_name=self.plural_name,
+            contributes_to=self.contributes_to,
+            publish_web_link=self.publish_web_link,
+            as_of=self.as_of, versioned_annotations=self.versioned_annotations)
         return interface
-    addClassAdvisor(mark_entry)
 
 
 def exported(field, *versioned_annotations, **kwparams):
@@ -352,9 +433,22 @@ def exported(field, *versioned_annotations, **kwparams):
     return field
 
 
+def _check_collection_default_content(name, interface):
+    """Check that the interface has a method providing the default content."""
+    tag = interface.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED, {})
+    if 'collection_default_content' not in tag:
+        raise TypeError(
+            "%s is missing a method tagged with @collection_default_content." %
+            name)
+
+
 def export_as_webservice_collection(entry_schema):
     """Mark the interface as exported on the web service as a collection.
 
+    This function does not work with Python 3.  Rather than calling this
+    within the class definition, decorate the class with
+    @exported_as_webservice_collection instead.
+
     :raises TypeError: if the interface doesn't have a method decorated with
         @collection_default_content.
     """
@@ -372,24 +466,46 @@ def export_as_webservice_collection(entry_schema):
     def mark_collection(interface):
         """Class advisor that tags the interface once it is created."""
         _check_interface('export_as_webservice_collection()', interface)
-
-        tag = interface.getTaggedValue(LAZR_WEBSERVICE_EXPORTED)
-        if 'collection_default_content' not in tag:
-            raise TypeError(
-                "export_as_webservice_collection() is missing a method "
-                "tagged with @collection_default_content.")
-
+        _check_collection_default_content(
+            'export_as_webservice_collection()', interface)
         annotate_exported_methods(interface)
         return interface
 
     addClassAdvisor(mark_collection)
 
 
+class exported_as_webservice_collection:
+    """Mark the interface as exported on the web service as a collection.
+
+    :raises TypeError: if the interface doesn't have a method decorated with
+        @collection_default_content.
+    """
+
+    def __init__(self, entry_schema):
+        if not IInterface.providedBy(entry_schema):
+            raise TypeError("entry_schema must be an interface.")
+        self.entry_schema = entry_schema
+
+    def __call__(self, interface):
+        _check_interface('exported_as_webservice_collection()', interface)
+        _check_collection_default_content(
+            'exported_as_webservice_collection()', interface)
+        tag = interface.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)
+        if tag is None:
+            tag = {}
+            interface.setTaggedValue(LAZR_WEBSERVICE_EXPORTED, tag)
+        tag['type'] = COLLECTION_TYPE
+        tag['collection_entry_schema'] = self.entry_schema
+        annotate_exported_methods(interface)
+        return interface
+
+
 class collection_default_content:
     """Decorates the method that provides the default values of a collection.
 
-    :raises TypeError: if not called from within an interface exported as a
-        collection, or if used more than once in the same interface.
+    :raises TypeError: if called from within an interface exported as
+        something other than a collection, or if used more than once for the
+        same version in the same interface.
     """
 
     def __init__(self, version=None, **params):
@@ -403,9 +519,20 @@ class collection_default_content:
         """
         _check_called_from_interface_def('@collection_default_content')
 
+        # We used to check that this decorator is being used from within an
+        # interface exported as a collection.  However, it isn't possible to
+        # do this when using the @exported_as_webservice_collection
+        # decorator rather than the export_as_webservice_collection function
+        # (which is based on class advice, and so cannot work in Python 3):
+        # that decorator isn't called until the rest of the interface has
+        # been defined, so it hasn't had a chance to set any tags on the
+        # interface by the time decorators of methods in that interface are
+        # called.  The best we can do is to check that we don't already
+        # positively know that the interface is exported as an entry
+        # instead.
         tags = _get_interface_tags()
-        tag = tags.get(LAZR_WEBSERVICE_EXPORTED)
-        if tag is None or tag['type'] != COLLECTION_TYPE:
+        tag = tags.setdefault(LAZR_WEBSERVICE_EXPORTED, {})
+        if 'type' in tag and tag['type'] != COLLECTION_TYPE:
             raise TypeError(
                 "@collection_default_content can only be used from within an "
                 "interface exported as a collection.")
@@ -490,8 +617,11 @@ class _method_annotator:
 
     The actual method will be wrapped in an IMethod specification once the
     Interface is complete. So we save the annotations in an attribute of the
-    method, and the class advisor invoked by export_as_webservice_entry() and
-    export_as_webservice_collection() will do the final tagging.
+    method, and either the class advisor invoked by
+    export_as_webservice_entry() and export_as_webservice_collection() or
+    the @exported_as_webservice_entry() and
+    @exported_as_webservice_collection() decorators will do the final
+    tagging.
     """
 
     def __call__(self, method):
@@ -1048,7 +1178,7 @@ def generate_entry_interfaces(interface, contributors=[], *versions):
     # we'll iterate over once for each version.
     tags_for_published_fields = []
     for iface in itertools.chain([interface], contributors):
-        for name, field in getFields(iface).items():
+        for name, field in getFieldsInOrder(iface):
             tag_stack = field.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)
             if tag_stack is None:
                 # This field is not published at all.
