@@ -2,6 +2,8 @@
 
 """Traversal rules for the LAZR example web service."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'BelowRootAbsoluteURL',
@@ -11,9 +13,7 @@ __all__ = [
     ]
 
 
-from urllib import unquote
-from zope.component import adapts
-from zope.interface import implements
+from six.moves.urllib.parse import unquote
 from zope.publisher.interfaces import IPublishTraverse, NotFound
 from zope.publisher.interfaces.browser import IDefaultBrowserLayer
 from zope.traversing.browser import absoluteURL, AbsoluteURL
@@ -21,7 +21,6 @@ from zope.traversing.browser import absoluteURL, AbsoluteURL
 import grokcore.component
 
 from lazr.restful.example.base.interfaces import ICookbookSet, IHasGet
-from lazr.restful.example.base.root import CookbookWebServiceObject
 from lazr.restful.simple import (
     RootResourceAbsoluteURL,
     SimulatedWebsiteRequest,
