@@ -2,6 +2,8 @@
 
 """Marshallers for fields used in HTTP resources."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'AbstractCollectionFieldMarshaller',
@@ -25,9 +27,9 @@ __all__ = [
 from datetime import datetime
 import pytz
 from StringIO import StringIO
-import urllib
 
 import simplejson
+from six.moves.urllib.parse import unquote
 
 from zope.datetime import (
     DateTimeError,
@@ -37,7 +39,7 @@ from zope.component import (
     getMultiAdapter,
     getUtility,
     )
-from zope.interface import implements
+from zope.interface import implementer
 from zope.publisher.interfaces import NotFound
 from zope.schema import Field
 from zope.security.proxy import removeSecurityProxy
@@ -111,7 +113,7 @@ class URLDereferencingMixin:
             or fragment is not None):
             raise NotFound(self, url, self.request)
 
-        path_parts = [urllib.unquote(part) for part in path.split('/')]
+        path_parts = [unquote(part) for part in path.split('/')]
         path_parts.pop(0)
         path_parts.reverse()
         request = config.createRequest(StringIO(), {'PATH_INFO': path})
@@ -120,12 +122,12 @@ class URLDereferencingMixin:
         return request.traverse(root)
 
 
+@implementer(IFieldMarshaller)
 class SimpleFieldMarshaller:
     """A marshaller that returns the same value it's served.
 
     This implementation is meant to be subclassed.
     """
-    implements(IFieldMarshaller)
 
     # Set this to type or tuple of types that the JSON value must be of.
     _type = None
@@ -156,18 +158,13 @@ class SimpleFieldMarshaller:
                 v = value
                 if isinstance(v, str):
                     v = v.decode('utf8') # assume utf8
-                # XXX gary 2009-03-28
-                # The use of the enclosing brackets is a hack to work around
-                # simplejson bug 43:
-                # http://code.google.com/p/simplejson/issues/detail?id=43
-                v = simplejson.loads(u'[%s]' % (v,))
+                elif not isinstance(v, unicode):
+                    v = unicode(v)
+                value = simplejson.loads(v)
             except (ValueError, TypeError):
                 # Pass the value as is. This saves client from having to encode
                 # strings.
                 pass
-            else:
-                # see comment about simplejson bug above
-                value = v[0]
         if value is None:
             return None
         return self._marshall_from_request(value)
@@ -543,9 +540,9 @@ class DictFieldMarshaller(SimpleFieldMarshaller):
                for key, val in value.items())
 
 
+@implementer(IUnmarshallingDoesntNeedValue)
 class CollectionFieldMarshaller(SimpleFieldMarshaller):
     """A marshaller for collection fields."""
-    implements(IUnmarshallingDoesntNeedValue)
 
     @property
     def representation_name(self):
