@@ -2,10 +2,13 @@
 
 """ZCML registration directives for the LAZR webservice framework."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = []
 
 
+import collections
 import inspect
 import itertools
 
@@ -54,6 +57,23 @@ from lazr.restful.utils import VersionedObject
 REGISTERED_ENTRIES = []
 REGISTERED_OPERATIONS = []
 
+# Keep a global mapping of each interface's contributors, registered by
+# contribute_to_interface and inspected by the entry generators.
+REGISTERED_CONTRIBUTORS = collections.defaultdict(list)
+
+
+def reset_globals():
+    del REGISTERED_ENTRIES[:]
+    del REGISTERED_OPERATIONS[:]
+    REGISTERED_CONTRIBUTORS.clear()
+
+try:
+    from zope.testing.cleanup import addCleanUp
+except ImportError:
+    pass
+else:
+    addCleanUp(reset_globals)
+
 
 class IRegisterDirective(Interface):
     """Directive to hook up webservice based on the declarations in a module.
@@ -62,8 +82,30 @@ class IRegisterDirective(Interface):
         title=u'Module which will be inspected for webservice declarations')
 
 
-def generate_and_register_entry_adapters(interface, info, contributors,
-                                         repository=None):
+def contribute_to_interface(interface, info):
+    tag = interface.getTaggedValue(LAZR_WEBSERVICE_EXPORTED)
+    for iface in tag['contributes_to']:
+        if iface.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED) is None:
+            raise AttemptToContributeToNonExportedInterface(
+                "Interface %s contributes to %s, which is not "
+                "exported." % (interface.__name__, iface.__name__))
+
+        # Check that no names conflict with the extended interface or
+        # its contributors.
+        contributors = REGISTERED_CONTRIBUTORS[iface]
+        names = {}
+        for iface in itertools.chain([iface, interface], contributors):
+            for name, f in iface.namesAndDescriptions(all=True):
+                if f.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED) is not None:
+                    L = names.setdefault(name, [])
+                    L.append(iface)
+        for name, interfaces in names.items():
+            if len(interfaces) > 1:
+                raise ConflictInContributingInterfaces(name, interfaces)
+        contributors.append(interface)
+
+
+def generate_and_register_entry_adapters(interface, info, repository=None):
     """Generate an entry adapter for every version of the web service.
 
     This code generates an IEntry subinterface for every version, each
@@ -75,6 +117,7 @@ def generate_and_register_entry_adapters(interface, info, contributors,
     # Get the list of versions.
     config = getUtility(IWebServiceConfiguration)
     versions = list(config.active_versions)
+    contributors = REGISTERED_CONTRIBUTORS[interface]
 
     # Generate an interface and an adapter for every version.
     web_interfaces = generate_entry_interfaces(
@@ -134,15 +177,13 @@ def ensure_correct_version_ordering(name, version_list):
     # Sort version_list according to the list of actual versions.
     # If the sorted list is different from the unsorted list, at
     # least one version is out of place.
-    def compare(x, y):
-        return cmp (actual_versions.index(x), actual_versions.index(y))
-    sorted_version_list = sorted(version_list, cmp=compare)
+    sorted_version_list = sorted(version_list, key=actual_versions.index)
     if sorted_version_list != version_list:
         bad_versions = '", "'.join(version_list)
         good_versions = '", "'.join(sorted_version_list)
         msg = ('Annotations on "%s" put an earlier version on top of a '
                'later version: "%s". The correct order is: "%s".')
-        raise AssertionError, msg % (name, bad_versions, good_versions)
+        raise AssertionError(msg % (name, bad_versions, good_versions))
 
 
 def register_adapter_for_version(factory, interface, version_name,
@@ -204,59 +245,14 @@ def find_exported_interfaces(module):
 
     :return: iterator of interfaces.
     """
+    try:
+        module_all = set(module.__all__)
+        is_exported_name = lambda name: name in module_all
+    except AttributeError:
+        is_exported_name = lambda name: not name.startswith('_')
     return (interface for name, interface
-            in inspect.getmembers(module, _is_exported_interface))
-
-
-def find_interfaces_and_contributors(module):
-    """Find the interfaces and its contributors marked for export.
-
-    Return a dictionary with interfaces as keys and their contributors as
-    values.
-    """
-    interfaces_with_contributors = {}
-    for interface in find_exported_interfaces(module):
-        if issubclass(interface, Exception):
-            # Exceptions can't have interfaces, so just store it in
-            # interfaces_with_contributors and move on.
-            interfaces_with_contributors.setdefault(interface, [])
-            continue
-
-        tag = interface.getTaggedValue(LAZR_WEBSERVICE_EXPORTED)
-        if tag.get('contributes_to'):
-            # Append this interface (which is a contributing interface) to the
-            # list of contributors of every interface it contributes to.
-            for iface in tag['contributes_to']:
-                if iface.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED) is None:
-                    raise AttemptToContributeToNonExportedInterface(
-                        "Interface %s contributes to %s, which is not "
-                        "exported." % (interface.__name__, iface.__name__))
-                    raise AssertionError('foo')
-                contributors = interfaces_with_contributors.setdefault(
-                    iface, [])
-                contributors.append(interface)
-        else:
-            # This is a regular interface, but one of its contributing
-            # interfaces may have been processed previously and in that case a
-            # key for it would already exist in interfaces_with_contributors;
-            # that's why we use setdefault.
-            interfaces_with_contributors.setdefault(interface, [])
-
-    # For every exported interface, check that none of its names are exported
-    # in more than one contributing interface.
-    for interface, contributors in interfaces_with_contributors.items():
-        if len(contributors) == 0:
-            continue
-        names = {}
-        for iface in itertools.chain([interface], contributors):
-            for name, f in iface.namesAndDescriptions(all=True):
-                if f.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED) is not None:
-                    L = names.setdefault(name, [])
-                    L.append(iface)
-        for name, interfaces in names.items():
-            if len(interfaces) > 1:
-                raise ConflictInContributingInterfaces(name, interfaces)
-    return interfaces_with_contributors
+            in inspect.getmembers(module, _is_exported_interface)
+            if is_exported_name(name))
 
 
 class ConflictInContributingInterfaces(Exception):
@@ -287,21 +283,32 @@ def register_webservice(context, module):
     if not inspect.ismodule(module):
         raise TypeError("module attribute must be a module: %s, %s" %
                         module, type(module))
-    interfaces_with_contributors = find_interfaces_and_contributors(module)
 
-    for interface, contributors in interfaces_with_contributors.items():
+    for interface in find_exported_interfaces(module):
         if issubclass(interface, Exception):
             register_exception_view(context, interface)
             continue
 
         tag = interface.getTaggedValue(LAZR_WEBSERVICE_EXPORTED)
         if tag['type'] == ENTRY_TYPE:
-            context.action(
-                discriminator=('webservice entry interface', interface),
-                    callable=generate_and_register_entry_adapters,
-                    args=(interface, context.info, contributors,
-                          REGISTERED_ENTRIES),
-                    )
+            if tag.get('contributes_to'):
+                # This isn't a standalone entry, so just register its
+                # contributions in the global map. This has to be done
+                # before entry and operation adapter generation, so we
+                # customise the order.
+                context.action(
+                    discriminator=('webservice entry contributor', interface),
+                        callable=contribute_to_interface,
+                        args=(interface, context.info),
+                        order=1,
+                        )
+            else:
+                context.action(
+                    discriminator=('webservice entry interface', interface),
+                        callable=generate_and_register_entry_adapters,
+                        args=(interface, context.info, REGISTERED_ENTRIES),
+                        order=2,
+                        )
         elif tag['type'] == COLLECTION_TYPE:
             for version in tag['collection_default_content'].keys():
                 factory = generate_collection_adapter(interface, version)
@@ -318,8 +325,9 @@ def register_webservice(context, module):
             raise AssertionError('Unknown export type: %s' % tag['type'])
         context.action(
             discriminator=('webservice versioned operations', interface),
-            args=(context, interface, contributors, REGISTERED_OPERATIONS),
-            callable=generate_and_register_webservice_operations)
+            args=(context, interface, REGISTERED_OPERATIONS),
+            callable=generate_and_register_webservice_operations,
+            order=2)
 
 @adapter(IProcessStarting)
 def webservice_sanity_checks(registration):
@@ -448,7 +456,7 @@ def _assert_interface_registered_for_version(
 
 
 def generate_and_register_webservice_operations(
-        context, interface, contributors, repository=None):
+        context, interface, repository=None):
     """Create and register adapters for all exported methods.
 
     Different versions of the web service may publish the same
@@ -468,9 +476,11 @@ def generate_and_register_webservice_operations(
                 no_mutator_operations_after+1]
         else:
             block_mutator_operations_as_of_version = None
+    registered_adapters = set()
+    block_mutator_operations = set()
 
     methods = interface.namesAndDescriptions(True)
-    for iface in contributors:
+    for iface in REGISTERED_CONTRIBUTORS[interface]:
         methods.extend(iface.namesAndDescriptions(True))
 
     for name, method in methods:
@@ -514,8 +524,6 @@ def generate_and_register_webservice_operations(
         # version.
         previous_operation_name = None
         previous_operation_provides = None
-        mutator_operation_needs_to_be_blocked = False
-        master_stack = tag
         for version, tag in tag.stack:
             if version is None:
                 this_version_index = 0
@@ -556,7 +564,8 @@ def generate_and_register_webservice_operations(
                         'Method "%s" contains annotations for version "%s", '
                         'even though it\'s not published in that version. '
                         'The bad annotations are: "%s".' % (
-                            method.__name__, version, '", "'.join(tag_names)))
+                            method.__name__, version,
+                            '", "'.join(sorted(tag_names))))
             else:
                 if tag['type'] == 'read_operation':
                     operation_provides = IResourceGETOperation
@@ -585,7 +594,6 @@ def generate_and_register_webservice_operations(
                     # This will save us from having to do another
                     # de-registration later.
                     factory = _mask_adapter_registration
-                    mutator_operation_needs_to_be_blocked = False
                 else:
                     factory = generate_operation_adapter(method, version)
 
@@ -600,6 +608,9 @@ def generate_and_register_webservice_operations(
                     _mask_adapter_registration, interface, version,
                     previous_operation_provides, previous_operation_name,
                     context.info)
+                registered_adapters.add(
+                    (previous_operation_name, version,
+                     previous_operation_provides))
 
             # If the operation exists in this version (ie. its name is
             # not None), register it using this version's name.
@@ -607,6 +618,8 @@ def generate_and_register_webservice_operations(
                 register_adapter_for_version(
                     factory, interface, version, operation_provides,
                     operation_name, context.info)
+                registered_adapters.add(
+                    (operation_name, version, operation_provides))
                 if (tag.get('is_mutator')
                     and block_mutator_operations_as_of_version != None):
                     defined_in = this_version_index
@@ -618,21 +631,25 @@ def generate_and_register_webservice_operations(
                         # mutators stop being named operations, and
                         # the operation registration for this mutator
                         # will need to be blocked.
-                        mutator_operation_needs_to_be_blocked = True
+                        block_mutator_operations.add(
+                            (operation_name,
+                             block_mutator_operations_as_of_version,
+                             operation_provides))
                 if repository is not None:
                     repository.append(VersionedObject(version, method))
             previous_operation_name = operation_name
             previous_operation_provides = operation_provides
 
-        if mutator_operation_needs_to_be_blocked:
+    for operation_key in block_mutator_operations:
+        if operation_key not in registered_adapters:
+            operation_name, version, operation_provides = operation_key
             # The operation was registered as a mutator, back in the
             # days when mutator operations were also named operations,
             # and it never got de-registered. De-register it now.
             register_adapter_for_version(
-                _mask_adapter_registration, interface,
-                block_mutator_operations_as_of_version,
-                previous_operation_provides, previous_operation_name,
-                context.info)
+                _mask_adapter_registration, interface, version,
+                operation_provides, operation_name, context.info)
+
 
 def _mask_adapter_registration(*args):
     """A factory function that stops an adapter lookup from succeeding.
