@@ -1,5 +1,7 @@
 """Simple implementations of various Zope and lazr.restful interfaces."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'BaseRepresentationCache',
@@ -17,18 +19,19 @@ __all__ = [
     ]
 
 import traceback
-import urllib
 
+from six.moves.urllib.parse import (
+    quote,
+    unquote,
+    )
 from zope.component import (
-    adapts, getMultiAdapter, getUtility, queryMultiAdapter)
-from zope.interface import Attribute, Interface, implements
+    adapter, getMultiAdapter, getUtility, queryMultiAdapter)
+from zope.interface import Attribute, Interface, implementer
 from zope.publisher.browser import BrowserRequest
 from zope.publisher.interfaces import IPublication, IPublishTraverse, NotFound
 from zope.publisher.publish import mapply
-from zope.proxy import sameProxiedObjects
 from zope.security.management import endInteraction, newInteraction
-from zope.traversing.browser import (
-    absoluteURL, AbsoluteURL as ZopeAbsoluteURL)
+from zope.traversing.browser import absoluteURL
 from zope.traversing.browser.interfaces import IAbsoluteURL
 from zope.traversing.browser.absoluteurl import _insufficientContext, _safe
 
@@ -46,19 +49,21 @@ from lazr.restful.interfaces import (
     IWebServiceLayer,
     )
 from lazr.restful.publisher import (
-    browser_request_to_web_service_request, WebServicePublicationMixin,
-    WebServiceRequestTraversal)
+    WebServicePublicationMixin,
+    WebServiceRequestTraversal,
+    )
 from lazr.restful.utils import (
-    get_current_browser_request, implement_from_dict,
-    tag_request_with_version_name)
+    implement_from_dict,
+    tag_request_with_version_name,
+    )
 
 
+@implementer(IPublication)
 class PublicationMixin(object):
     """A very simple implementation of `IPublication`.
 
     The object passed to the constructor is returned by getApplication().
     """
-    implements(IPublication)
 
     def __init__(self, application):
         """Create the test publication.
@@ -148,6 +153,7 @@ class Publication(WebServicePublicationMixin, PublicationMixin):
     pass
 
 
+@implementer(ITraverseWithGet)
 class TraverseWithGet(object):
     """An implementation of `IPublishTraverse` that uses the get() method.
 
@@ -162,11 +168,10 @@ class TraverseWithGet(object):
     collections or fields) always happens with
     WebServiceRequestTraversal.
     """
-    implements(ITraverseWithGet)
 
     def publishTraverse(self, request, name):
         """See `IPublishTraverse`."""
-        name = urllib.unquote(name)
+        name = unquote(name)
         value = self.get(request, name)
         if value is None:
             raise NotFound(self, name)
@@ -177,11 +182,12 @@ class TraverseWithGet(object):
         raise NotImplementedError
 
 
+@implementer(IWebServiceLayer)
 class Request(WebServiceRequestTraversal, BrowserRequest):
     """A web service request with no special features."""
-    implements(IWebServiceLayer)
 
 
+@implementer(IWebBrowserOriginatingRequest)
 class SimulatedWebsiteRequest(BrowserRequest):
     """A (simulated) request to a website as opposed to a web service.
 
@@ -189,7 +195,6 @@ class SimulatedWebsiteRequest(BrowserRequest):
     other class that implements IWebBrowserOriginatingRequest), you
     can take advantage of the web_url feature.
     """
-    implements(IWebBrowserOriginatingRequest)
 
 
 class RootResource(ServiceRootResource, TraverseWithGet):
@@ -267,12 +272,11 @@ class RootResource(ServiceRootResource, TraverseWithGet):
         return ({}, {})
 
 
+@implementer(IAbsoluteURL)
+@adapter(ServiceRootResource, WebServiceRequestTraversal)
 class RootResourceAbsoluteURL:
     """A basic implementation of `IAbsoluteURL` for a root resource."""
 
-    implements(IAbsoluteURL)
-    adapts(ServiceRootResource, WebServiceRequestTraversal)
-
     def __init__(self, context, request):
         """Initialize with respect to a context and request."""
         config = getUtility(IWebServiceConfiguration)
@@ -338,6 +342,8 @@ class IMultiplePathPartLocation(Interface):
         'corresponds to the URL path "foo/bar"')
 
 
+@implementer(IAbsoluteURL)
+@adapter(IMultiplePathPartLocation, WebServiceRequestTraversal)
 class MultiplePathPartAbsoluteURL:
     """Generate a URL for an IMultiplePathPartLocation.
 
@@ -348,8 +354,6 @@ class MultiplePathPartAbsoluteURL:
     as the IAbsoluteURL adapter for IMultiplePathPartLocation objects
     and web service requests.
     """
-    implements(IAbsoluteURL)
-    adapts(IMultiplePathPartLocation, WebServiceRequestTraversal)
 
     def __init__(self, context, request):
         """Initialize with respect to a context and request."""
@@ -369,12 +373,12 @@ class MultiplePathPartAbsoluteURL:
         if not hasattr(parts, '__iter__'):
             raise TypeError("Expected an iterable for __path_parts__.")
 
-        escaped_parts = [urllib.quote(part.encode('utf-8'), _safe)
-                         for part in parts]
+        escaped_parts = [quote(part.encode('utf-8'), _safe) for part in parts]
         return start_url + "/" + "/".join(escaped_parts)
     __call__ = __str__
 
 
+@implementer(IRepresentationCache)
 class BaseRepresentationCache(object):
     """A useful base class for representation caches.
 
@@ -388,7 +392,6 @@ class BaseRepresentationCache(object):
     You can also implement set_by_key() and get_by_key(), which also
     take the return value of key_for(), instead of set() and get().
     """
-    implements(IRepresentationCache)
 
     DO_NOT_CACHE = object()
 
