@@ -1,28 +1,58 @@
 # Copyright 2008 Canonical Ltd.  All rights reserved.
 #
+# Portions from epydoc, licensed as follows:
+#
+#     Copyright (C) 2006 Edward Loper.
+#
+#     Permission is hereby granted, free of charge, to any person
+#     obtaining a copy of this software and any associated documentation
+#     files (the "Software"), to deal in the Software without
+#     restriction, including without limitation the rights to use, copy,
+#     modify, merge, publish, distribute, sublicense, and/or sell copies
+#     of the Software, and to permit persons to whom the Software is
+#     furnished to do so, subject to the following conditions:
+#
+#     The above copyright notice and this permission notice shall be
+#     included in all copies or substantial portions of the Software.
+#
+#     The software is provided "as is", without warranty of any
+#     kind, express or implied, including but not limited to the
+#     warranties of merchantability, fitness for a particular purpose
+#     and noninfringement. In no event shall the authors or copyright
+#     holders be liable for any claim, damages or other liability,
+#     whether in an action of contract, tort or otherwise, arising from,
+#     out of or in connection with the software or the use or other
+#     dealings in the software.
+
 """Implementation of the ws: namespace in TALES."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 
-all = ['entry_adapter_for_schema']
-
 import operator
+import re
 import simplejson
 import textwrap
-import urllib
 
-from epydoc.markup import DocstringLinker
-from epydoc.markup.restructuredtext import (
-    _DocumentPseudoWriter,
-    _EpydocReader,
-    ParsedRstDocstring,
-    )
 from docutils import io
 from docutils.core import Publisher
-
+from docutils.frontend import OptionParser
+from docutils.nodes import (
+    Node,
+    SkipNode,
+    )
+from docutils.readers.standalone import Reader as StandaloneReader
+from docutils.utils import new_document
+from docutils.writers.html4css1 import (
+    HTMLTranslator,
+    Writer as HTMLWriter,
+    )
+from docutils.writers.null import Writer as NullWriter
+from six.moves.urllib.parse import quote
 from zope.component import (
-    adapts, getGlobalSiteManager, getUtility, queryMultiAdapter)
-from zope.interface import implements
+    adapter, getGlobalSiteManager, getUtility, queryMultiAdapter)
+from zope.interface import implementer
 from zope.interface.interfaces import IInterface
 from zope.schema import getFields
 from zope.schema.interfaces import (
@@ -46,32 +76,56 @@ from lazr.restful.interfaces import (
     ICollection, ICollectionField, IEntry, IJSONRequestCache,
     IReference, IResourceDELETEOperation, IResourceGETOperation,
     IResourceOperation, IResourcePOSTOperation, IScopedCollection,
-    ITopLevelEntryLink, IWebServiceClientRequest, IWebServiceConfiguration,
+    ITopLevelEntryLink, IWebServiceConfiguration,
     IWebServiceVersion, LAZR_WEBSERVICE_NAME)
 from lazr.restful.utils import (get_current_web_service_request,
     is_total_size_link_active)
 
 
-class WadlDocstringLinker(DocstringLinker):
-    """DocstringLinker used during WADL geneneration.
+class _ParseError(Exception):
+    """An error generated while parsing a docstring."""
 
-    epydoc uses this object to turn index and identifier references
-    like `DocstringLinker` into an appropriate markup in the output
-    format.
+    def __init__(self, descr, linenum=None):
+        self._descr = descr
+        self._linenum = linenum
 
-    We don't want to generate links in the WADL file so we basically
-    return the identifier without any special linking markup.
+    def __str__(self):
+        if self._linenum is not None:
+            return 'Line %s: %s' % (self._linenum, self._descr)
+        else:
+            return self._descr
+
+
+class _ErrorAccumulatingReader(StandaloneReader):
+    """
+    A reader that captures all errors that are generated by parsing,
+    and appends them to a list.
     """
 
-    def translate_identifier_xref(self, identifier, label=None):
-        """See `DocstringLinker`."""
-        if label:
-            return label
-        return identifier
+    def __init__(self, errors):
+        self._errors = errors
+        StandaloneReader.__init__(self)
 
-    def translate_indexterm(self, indexterm):
-        """See `DocstringLinker`."""
-        return indexterm
+    def new_document(self):
+        document = new_document(self.source.source_path, self.settings)
+        # Capture all warning messages.
+        document.reporter.attach_observer(self.report)
+        # These are used so we know how to encode warning messages:
+        self._encoding = document.reporter.encoding
+        self._error_handler = document.reporter.error_handler
+        # Return the new document.
+        return document
+
+    def report(self, error):
+        try:
+            linenum = int(error['line'])
+        except ValueError:
+            linenum = None
+
+        msg = ''.join([c.astext().encode(self._encoding, self._error_handler)
+                       for c in error])
+
+        self._errors.append(_ParseError(msg, linenum))
 
 
 class _PydocParser:
@@ -80,10 +134,12 @@ class _PydocParser:
     def __init__(self):
         # Set up the instance we'll be using to render docstrings.
         self.errors = []
-        self.writer = _DocumentPseudoWriter()
-        self.publisher = Publisher(_EpydocReader(self.errors),
+        self.writer = NullWriter()
+        self.publisher = Publisher(
+            _ErrorAccumulatingReader(self.errors),
             writer=self.writer,
-            source_class=io.StringInput)
+            source_class=io.StringInput,
+            destination_class=io.NullOutput)
         self.publisher.set_components('standalone', 'restructuredtext',
             'pseudoxml')
         settings_overrides={
@@ -95,7 +151,6 @@ class _PydocParser:
             settings_overrides, None)
         self.publisher.set_destination()
 
-
     def parse_docstring(self, docstring, errors):
         """Parse a docstring for eventual transformation into HTML
 
@@ -111,12 +166,86 @@ class _PydocParser:
         self.publisher.publish()
         # Move any errors into the caller-provided list.
         errors[:] = self.errors[:]
-        return ParsedRstDocstring(self.writer.document)
+        return self.writer.document
 
 
 _PYDOC_PARSER = _PydocParser()
 
 
+class _PydocHTMLTranslator(HTMLTranslator):
+    settings = None
+
+    def __init__(self, document):
+        # Set the document's settings.
+        if self.settings is None:
+            settings = OptionParser([HTMLWriter()]).get_default_values()
+            self.__class__.settings = settings
+        document.settings = self.settings
+
+        # Call the parent constructor.
+        HTMLTranslator.__init__(self, document)
+
+    # Handle interpreted text (crossreferences)
+    def visit_title_reference(self, node):
+        self.body.append(self.encode(node.astext()))
+        raise SkipNode()
+
+    def should_be_compact_paragraph(self, node):
+        if self.document.children == [node]:
+            return True
+        else:
+            return HTMLTranslator.should_be_compact_paragraph(self, node)
+
+    def visit_document(self, node):
+        pass
+
+    def depart_document(self, node):
+        pass
+
+    def starttag(self, node, tagname, suffix='\n', **attributes):
+        """
+        This modified version of starttag makes a few changes to HTML
+        tags, to prevent conflicts.  In particular:
+          - existing class attributes are prefixed with C{'rst-'}
+          - existing names are prefixed with C{'rst-'}
+          - hrefs starting with C{'#'} are prefixed with C{'rst-'}
+          - hrefs not starting with C{'#'} are given target='_top'
+          - all headings (C{<hM{n}>}) are given the css class C{'heading'}
+        """
+        # Get the list of all attribute dictionaries we need to munge.
+        attr_dicts = [attributes]
+        if isinstance(node, Node):
+            attr_dicts.append(node.attributes)
+        if isinstance(node, dict):
+            attr_dicts.append(node)
+        # Munge each attribute dictionary.  Unfortunately, we need to
+        # iterate through attributes one at a time because some
+        # versions of docutils don't case-normalize attributes.
+        for attr_dict in attr_dicts:
+            for (key, val) in attr_dict.items():
+                # Prefix all CSS classes with "rst-"; and prefix all
+                # names with "rst-" to avoid conflicts.
+                if key.lower() in ('class', 'id', 'name'):
+                    attr_dict[key] = 'rst-%s' % val
+                elif key.lower() in ('classes', 'ids', 'names'):
+                    attr_dict[key] = ['rst-%s' % cls for cls in val]
+                elif key.lower() == 'href':
+                    if attr_dict[key][:1] == '#':
+                        attr_dict[key] = '#rst-%s' % attr_dict[key][1:]
+                    else:
+                        # If it's an external link, open it in a new
+                        # page.
+                        attr_dict['target'] = '_top'
+
+        # For headings, use class="heading"
+        if re.match(r'^h\d+$', tagname):
+            attributes['class'] = ' '.join(
+                [attributes.get('class', ''), 'heading']).strip()
+
+        return HTMLTranslator.starttag(
+            self, node, tagname, suffix, **attributes)
+
+
 WADL_DOC_TEMPLATE = (
     '<wadl:doc xmlns="http://www.w3.org/1999/xhtml">\n%s\n</wadl:doc>')
 
@@ -138,13 +267,15 @@ def generate_wadl_doc(doc):
         raise AssertionError(
             "Invalid docstring %s:\n %s" % (doc, "\n ".join(messages)))
 
-    return WADL_DOC_TEMPLATE % parsed.to_html(WadlDocstringLinker())
+    visitor = _PydocHTMLTranslator(parsed)
+    parsed.walkabout(visitor)
+    return WADL_DOC_TEMPLATE % ''.join(visitor.body)
 
 
+@implementer(IPathAdapter)
+@adapter(IBrowserRequest)
 class WebServiceRequestAPI:
     """Namespace for web service functions related to a website request."""
-    implements(IPathAdapter)
-    adapts(IBrowserRequest)
 
     def __init__(self, request):
         """Initialize with respect to a request."""
@@ -243,7 +374,7 @@ class WadlCollectionResourceAPI(WadlResourceAPI):
                 relationship_name = self.context.relationship.__name__
             return (absoluteURL(self.context.context,
                                 get_current_web_service_request()) + '/' +
-                    urllib.quote(relationship_name))
+                    quote(relationship_name))
         else:
             return super(WadlCollectionResourceAPI, self).url
 
@@ -603,7 +734,7 @@ class WadlFieldAPI(RESTUtilityBase):
         try:
             return EntryAdapterUtility.forSchemaInterface(
                 schema, get_current_web_service_request())
-        except UnknownEntryAdapter, e:
+        except UnknownEntryAdapter as e:
             e.whence = (
                 'Encountered as a result of the entry interface %r, field %r.'
                 % (self.field.interface, self.field.getName()))
