@@ -2,13 +2,15 @@
 
 """Helper functions for testing TALES expressions."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 
 __all__ = [
     'test_tales',
     ]
 
-from zope.app.pagetemplate.engine import TrustedEngine
+from zope.pagetemplate.engine import TrustedEngine
 
 
 class Context:
