@@ -2,13 +2,13 @@
 
 """Testing helpers for webservice unit tests."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 __all__ = [
     'create_web_service_request',
     'DummyAbsoluteURL',
     'DummyRootResourceURL',
-    'ExampleWebServiceTestCaller',
-    'ExampleWebServicePublication',
     'FakeRequest',
     'FakeResponse',
     'IGenericEntry',
@@ -16,29 +16,27 @@ __all__ = [
     'pprint_entry',
     'simple_renderer',
     'WebServiceTestCase',
-    'WebServiceTestConfiguration'
+    'WebServiceTestConfiguration',
     'WebServiceTestPublication',
-    'WebServiceTestRequest',
-    'TestPublication',
     ]
 
 from cStringIO import StringIO
-import os
-import traceback
 import simplejson
 import sys
 from types import ModuleType
 import unittest
-import urllib
 
-from urlparse import urljoin
 import wsgi_intercept
 
+from six.moves.urllib.parse import (
+    quote,
+    urlencode,
+    urljoin,
+    )
 from zope.component import (
-    adapts, getGlobalSiteManager, getUtility, queryMultiAdapter)
+    adapter, getGlobalSiteManager, getUtility)
 from zope.configuration import xmlconfig
-from zope.interface import alsoProvides, implements, Interface
-from zope.publisher.browser import BrowserRequest
+from zope.interface import alsoProvides, implementer, Interface
 from zope.publisher.interfaces.http import IHTTPApplicationRequest
 from zope.publisher.paste import Application
 from zope.proxy import ProxyBase
@@ -49,20 +47,16 @@ from zope.traversing.browser.interfaces import IAbsoluteURL
 
 from lazr.restful.declarations import (
     collection_default_content, exported,
-    export_as_webservice_collection, export_as_webservice_entry,
+    exported_as_webservice_collection, exported_as_webservice_entry,
     export_read_operation, operation_parameters)
 from lazr.restful.interfaces import (
     IServiceRootResource, IWebServiceClientRequest,
     IWebServiceConfiguration, IWebServiceLayer, IWebServiceVersion)
-from lazr.restful.publisher import (
-    WebServicePublicationMixin, WebServiceRequestTraversal)
 from lazr.restful.simple import (
     BaseWebServiceConfiguration, Publication, Request,
     RootResourceAbsoluteURL, ServiceRootResource)
 from lazr.restful.utils import tag_request_with_version_name
 
-from lazr.uri import URI
-
 
 def create_web_service_request(path_info, method='GET', body=None,
                                environ=None, hostname=None):
@@ -134,11 +128,11 @@ class FakeResponse:
         self.result = result
 
 
+@implementer(IHTTPApplicationRequest, IWebServiceLayer)
 class FakeRequest:
     """Simple request object for testing purpose."""
     # IHTTPApplicationRequest makes us eligible for
     # get_current_browser_request()
-    implements(IHTTPApplicationRequest, IWebServiceLayer)
 
     def __init__(self, traversed=None, stack=None, version=None):
         if version is None:
@@ -187,7 +181,7 @@ def pprint_entry(json_body):
     """
     for key, value in sorted(json_body.items()):
         if key != 'http_etag':
-            print '%s: %r' % (key, value)
+            print('%s: %r' % (key, value))
 
 
 def pprint_collection(json_body):
@@ -196,11 +190,11 @@ def pprint_collection(json_body):
         if key == 'total_size_link':
             continue
         if key != 'entries':
-            print '%s: %r' % (key, value)
-    print '---'
+            print('%s: %r' % (key, value))
+    print('---')
     for entry in json_body['entries']:
         pprint_entry(entry)
-        print '---'
+        print('---')
 
 
 class WebServiceTestPublication(Publication):
@@ -342,7 +336,7 @@ class WebServiceCaller:
         for key, value in args.items():
             if not isinstance(value, basestring):
                 args[key] = simplejson.dumps(value)
-        return urllib.urlencode(args)
+        return urlencode(args)
 
     def named_get(self, path_or_url, operation_name, headers=None,
                   api_version=None, **kwargs):
@@ -371,7 +365,7 @@ class WebServiceCaller:
         """
         if not isinstance(value, basestring):
             value = simplejson.dumps(value)
-        return urllib.quote(value)
+        return quote(value)
 
     def _make_request_with_entity_body(self, path, method, media_type, data,
                                        headers, api_version):
@@ -425,9 +419,9 @@ class WebServiceResponseWrapper(ProxyBase):
             raise ValueError(self.body)
 
 
+@implementer(IWebServiceConfiguration)
 class WebServiceTestConfiguration(BaseWebServiceConfiguration):
     """A simple configuration for tests that need a running web service."""
-    implements(IWebServiceConfiguration)
     show_tracebacks = False
     active_versions = ['1.0', '2.0']
     hostname = "webservice_test"
@@ -552,12 +546,12 @@ class WebServiceTestCase(TestCaseWithWebServiceFixtures):
         """)
 
 
+@exported_as_webservice_entry()
 class IGenericEntry(Interface):
     """A simple, reusable entry interface for use in tests.
 
     The entry publishes one field and one named operation.
     """
-    export_as_webservice_entry()
 
     # pylint: disable-msg=E0213
     a_field = exported(
@@ -575,9 +569,9 @@ class IGenericEntry(Interface):
         """
 
 
+@exported_as_webservice_collection(IGenericEntry)
 class IGenericCollection(Interface):
     """A simple collection containing `IGenericEntry`, for use in tests."""
-    export_as_webservice_collection(IGenericEntry)
 
     # pylint: disable-msg=E0211
     @collection_default_content()
@@ -585,17 +579,17 @@ class IGenericCollection(Interface):
         """Returns all the entries."""
 
 
+@implementer(IServiceRootResource)
 class DummyRootResource:
     """A root resource that does nothing."""
-    implements(IServiceRootResource)
 
     def getTopLevelPublications(self, request):
         return {}
 
 
+@implementer(IAbsoluteURL)
 class DummyAbsoluteURL:
     """Implements IAbsoluteURL for when you don't care what the URL is."""
-    implements(IAbsoluteURL)
     URL = 'http://dummyurl/'
 
     def __init__(self, *args):
@@ -607,9 +601,9 @@ class DummyAbsoluteURL:
     __call__ = __str__
 
 
+@adapter(DummyRootResource, IWebServiceClientRequest)
 class DummyRootResourceURL(DummyAbsoluteURL):
     """A dummy IAbsoluteURL implementation for a DummyRootResource."""
-    adapts(DummyRootResource, IWebServiceClientRequest)
 
     URL = 'http://dummyurl'
 
