@@ -2,19 +2,23 @@
 
 """Unit tests for the conversion of interfaces into a web service."""
 
-from zope.configuration.config import ConfigurationExecutionError
+from __future__ import absolute_import, print_function
+
+import sys
+
+import testtools
 from zope.component import (
-    adapts,
+    adapter,
     getMultiAdapter,
     getSiteManager,
     getUtility,
     )
-from zope.component.interfaces import ComponentLookupError
 from zope.interface import (
     Attribute,
-    implements,
+    implementer,
     Interface,
     )
+from zope.interface.interfaces import ComponentLookupError
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.schema import (
     Int,
@@ -32,12 +36,17 @@ from zope.security.management import (
 from lazr.restful.declarations import (
     accessor_for,
     call_with,
+    collection_default_content,
+    error_status,
+    export_as_webservice_collection,
     export_as_webservice_entry,
-    exported,
     export_read_operation,
     export_write_operation,
+    exported,
+    exported_as_webservice_entry,
     generate_entry_interfaces,
     generate_operation_adapter,
+    LAZR_WEBSERVICE_EXPORTED,
     LAZR_WEBSERVICE_NAME,
     mutator_for,
     operation_for_version,
@@ -58,7 +67,8 @@ from lazr.restful.marshallers import SimpleFieldMarshaller
 from lazr.restful.metazcml import (
     AttemptToContributeToNonExportedInterface,
     ConflictInContributingInterfaces,
-    find_interfaces_and_contributors,
+    contribute_to_interface,
+    find_exported_interfaces,
     generate_and_register_webservice_operations,
     )
 from lazr.restful._resource import (
@@ -66,10 +76,7 @@ from lazr.restful._resource import (
     EntryResource,
     )
 from lazr.restful.testing.webservice import TestCaseWithWebServiceFixtures
-from lazr.restful.testing.helpers import (
-    create_test_module,
-    register_test_module,
-    )
+from lazr.restful.testing.helpers import register_test_module
 from lazr.restful.utils import VersionedObject
 
 
@@ -136,8 +143,8 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
         # There can only be one accessor defined for a given attribute
         # at a given version.
         def declare_too_many_accessors():
+            @exported_as_webservice_entry(contributes_to=[IProduct])
             class IHasTooManyAccessors(Interface):
-                export_as_webservice_entry(contributes_to=[IProduct])
                 needs_an_accessor = exported(
                     TextLine(title=u'This needs an accessor', readonly=True))
 
@@ -240,14 +247,16 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
     def test_redacted_fields_for_empty_entry(self):
         # An entry which doesn't export any fields/operations will have no
         # redacted fields.
+        @exported_as_webservice_entry()
         class IEmpty(Interface):
-            export_as_webservice_entry()
+            pass
+        @implementer(IEmpty)
         class Empty:
-            implements(IEmpty)
+            pass
         register_test_module('testmod', IEmpty)
         entry_resource = EntryResource(Empty(), self.one_zero_request)
-        self.assertEquals({}, entry_resource.entry._orig_interfaces)
-        self.assertEquals([], entry_resource.redacted_fields)
+        self.assertEqual({}, entry_resource.entry._orig_interfaces)
+        self.assertEqual([], entry_resource.redacted_fields)
 
     def test_redacted_fields_with_no_permission_checker(self):
         # When looking up an entry's redacted_fields, we take into account the
@@ -256,7 +265,7 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
         register_test_module(
             'testmod', IBranch, IProduct, IHasBugs, IHasBranches)
         entry_resource = EntryResource(self.product, self.one_zero_request)
-        self.assertEquals([], entry_resource.redacted_fields)
+        self.assertEqual([], entry_resource.redacted_fields)
 
     def test_redacted_fields_with_permission_checker(self):
         # When looking up an entry's redacted_fields for an object which is
@@ -270,24 +279,24 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
                 self.product,
                 checker=MultiChecker([(IProduct, 'zope.Public')]))
             entry_resource = EntryResource(secure_product, self.one_zero_request)
-            self.assertEquals([], entry_resource.redacted_fields)
+            self.assertEqual([], entry_resource.redacted_fields)
         finally:
             endInteraction()
 
     def test_duplicate_contributed_attributes(self):
         # We do not allow a given attribute to be contributed to a given
         # interface by more than one contributing interface.
-        testmod = create_test_module(
-            'testmod', IBranch, IProduct, IHasBugs, IHasBugs2)
+        contribute_to_interface(IHasBugs, {})
         self.assertRaises(
             ConflictInContributingInterfaces,
-            find_interfaces_and_contributors, testmod)
+            contribute_to_interface, IHasBugs2, {})
 
     def test_contributing_interface_not_exported(self):
         # Contributing interfaces are not exported by themselves -- they only
         # contribute their exported fields/operations to other entries.
+        @implementer(IHasBranches)
         class DummyHasBranches:
-            implements(IHasBranches)
+            pass
         dummy = DummyHasBranches()
         register_test_module('testmod', IBranch, IProduct, IHasBranches)
         self.assertRaises(
@@ -298,21 +307,20 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
         # A contributing interface can only contribute to exported interfaces.
         class INotExported(Interface):
             pass
+        @exported_as_webservice_entry(contributes_to=[INotExported])
         class IContributor(Interface):
-            export_as_webservice_entry(contributes_to=[INotExported])
             title = exported(TextLine(title=u'The project title'))
-        testmod = create_test_module('testmod', IContributor, INotExported)
         self.assertRaises(
             AttemptToContributeToNonExportedInterface,
-            find_interfaces_and_contributors, testmod)
+            contribute_to_interface, IContributor, {})
 
     def test_duplicate_contributed_methods(self):
         # We do not allow a given method to be contributed to a given
         # interface by more than one contributing interface.
-        testmod = create_test_module('testmod', IProduct, IHasBugs, IHasBugs3)
+        contribute_to_interface(IHasBugs, {})
         self.assertRaises(
             ConflictInContributingInterfaces,
-            find_interfaces_and_contributors, testmod)
+            contribute_to_interface, IHasBugs3, {})
 
     def test_ConflictInContributingInterfaces(self):
         # The ConflictInContributingInterfaces exception states what are the
@@ -320,7 +328,7 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
         e = ConflictInContributingInterfaces('foo', [IHasBugs, IHasBugs2])
         expected_msg = ("'foo' is exported in more than one contributing "
                         "interface: IHasBugs, IHasBugs2")
-        self.assertEquals(str(e), expected_msg)
+        self.assertEqual(str(e), expected_msg)
 
     def test_type_name(self):
         # Even though the generated adapters will contain stuff from various
@@ -332,8 +340,214 @@ class ContributingInterfacesTestCase(TestCaseWithWebServiceFixtures):
             'product', EntryAdapterUtility(adapter.__class__).singular_type)
 
 
+class TestExportAsWebserviceEntry(testtools.TestCase):
+    """Tests for export_as_webservice_entry."""
+
+    def setUp(self):
+        super(TestExportAsWebserviceEntry, self).setUp()
+        if sys.version_info[0] >= 3:
+            self.skipTest(
+                'export_as_webservice_entry is only supported on Python 2')
+
+    def test_works_on_interface(self):
+        # export_as_webservice_entry works on an interface.
+        class IFoo(Interface):
+            export_as_webservice_entry()
+
+        self.assertTrue(
+            IFoo.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)['exported'])
+
+    def test_requires_interface(self):
+        # export_as_webservice_entry can only be used on Interface.
+        def export_non_interface():
+            class NotAnInterface(object):
+                export_as_webservice_entry()
+
+        exception = self.assertRaises(TypeError, export_non_interface)
+        self.assertEqual(
+            'export_as_webservice_entry() can only be used on an interface.',
+            str(exception))
+
+    def test_must_be_within_interface_definition(self):
+        # export_as_webservice_entry can only be used from within a
+        # interface definition.
+        exception = self.assertRaises(TypeError, export_as_webservice_entry)
+        self.assertEqual(
+            'export_as_webservice_entry() can only be used from within an '
+            'interface definition.',
+            str(exception))
+
+
+class TestExportAsWebserviceCollection(testtools.TestCase):
+    """Tests for export_as_webservice_collection."""
+
+    def setUp(self):
+        super(TestExportAsWebserviceCollection, self).setUp()
+        if sys.version_info[0] >= 3:
+            self.skipTest(
+                'export_as_webservice_collection is only supported on '
+                'Python 2')
+
+    def test_works_on_interface_with_tagged_method(self):
+        # export_as_webservice_collection works on an interface that has an
+        # entry schema and a method tagged with @collection_default_content.
+        class IFoo(Interface):
+            export_as_webservice_collection(Interface)
+
+            @collection_default_content()
+            def getAll():
+                pass
+
+        self.assertEqual(
+            {None: ('getAll', {})},
+            IFoo.queryTaggedValue(LAZR_WEBSERVICE_EXPORTED)[
+                'collection_default_content'])
+
+    def test_requires_entry_schema(self):
+        # export_as_webservice_collection requires an entry schema.
+        def export_without_entry_schema():
+            class MissingEntrySchema(Interface):
+                export_as_webservice_collection()
+
+        exception = self.assertRaises(TypeError, export_without_entry_schema)
+        self.assertEqual(
+            'export_as_webservice_collection() takes exactly 1 argument (0 '
+            'given)',
+            str(exception))
+
+    def test_requires_entry_schema_as_interface(self):
+        # export_as_webservice_collection requires the entry schema to be an
+        # interface.
+        def export_with_invalid_entry_schema():
+            class InvalidEntrySchema(Interface):
+                export_as_webservice_collection('not an interface')
+
+        exception = self.assertRaises(
+            TypeError, export_with_invalid_entry_schema)
+        self.assertEqual('entry_schema must be an interface.', str(exception))
+
+    def test_requires_tagged_method(self):
+        # export_as_webservice_collection can only be used on a collection
+        # that has a method marked as exporting the default content.
+        def export_missing_default_content():
+            class MissingDefaultContent(Interface):
+                export_as_webservice_collection(Interface)
+
+        exception = self.assertRaises(
+            TypeError, export_missing_default_content)
+        self.assertEqual(
+            'export_as_webservice_collection() is missing a method tagged '
+            'with @collection_default_content.',
+            str(exception))
+
+    def test_refuses_multiple_tagged_methods(self):
+        # export_as_webservice_collection cannot be used on a collection
+        # that has multiple methods marked as exporting the default content.
+        def export_two_default_content():
+            class TwoDefaultContent(Interface):
+                export_as_webservice_collection(Interface)
+
+                @collection_default_content()
+                def getAll1():
+                    """A first getAll()."""
+
+                @collection_default_content()
+                def getAll2():
+                    """Another getAll()."""
+
+        exception = self.assertRaises(TypeError, export_two_default_content)
+        self.assertEqual(
+            "Only one method can be marked with @collection_default_content "
+            "for version '(earliest version)'.",
+            str(exception))
+
+    def test_requires_interface(self):
+        # export_as_webservice_collection can only be used on Interface.
+        def export_non_interface():
+            class NotAnInterface(object):
+                export_as_webservice_collection(Interface)
+
+        exception = self.assertRaises(TypeError, export_non_interface)
+        self.assertEqual(
+            'export_as_webservice_collection() can only be used on an '
+            'interface.',
+            str(exception))
+
+    def test_must_be_within_interface_definition(self):
+        # export_as_webservice_collection can only be used from within an
+        # interface definition.
+        exception = self.assertRaises(
+            TypeError, export_as_webservice_collection, Interface)
+        self.assertEqual(
+            'export_as_webservice_collection() can only be used from within '
+            'an interface definition.',
+            str(exception))
+
+
+class NotExportedException(Exception):
+    pass
+
+
+@error_status(400)
+class ExportedExceptionOne(Exception):
+    pass
+
+
+@error_status(400)
+class ExportedExceptionTwo(Exception):
+    pass
+
+
+class INotExported(Interface):
+    pass
+
+
+@exported_as_webservice_entry()
+class IExported(Interface):
+    pass
+
+
+class TestFindExportedInterfaces(testtools.TestCase):
+    """Tests for find_exported_interfaces."""
+
+    def assertExportsNames(self, module, names):
+        self.assertEqual(
+            sorted(names),
+            sorted(
+                iface.__name__ for iface in find_exported_interfaces(module)))
+
+    def test_finds_exported_exceptions(self):
+        class Module(object):
+            NotExportedException_copy = NotExportedException
+            ExportedExceptionOne_copy = ExportedExceptionOne
+
+        self.assertExportsNames(Module, ['ExportedExceptionOne'])
+
+    def test_finds_exported_interfaces(self):
+        class Module(object):
+            INotExported_copy = INotExported
+            IExported_copy = IExported
+
+        self.assertExportsNames(Module, ['IExported'])
+
+    def test_honours_all(self):
+        class Module(object):
+            __all__ = ['ExportedExceptionOne_copy']
+            ExportedExceptionOne_copy = ExportedExceptionOne
+            ExportedExceptionTwo_copy = ExportedExceptionTwo
+
+        self.assertExportsNames(Module, ['ExportedExceptionOne'])
+
+    def test_skips_leading_underscore_without_all(self):
+        class Module(object):
+            ExportedExceptionOne_copy = ExportedExceptionOne
+            _ExportedExceptionTwo_copy = ExportedExceptionTwo
+
+        self.assertExportsNames(Module, ['ExportedExceptionOne'])
+
+
+@exported_as_webservice_entry()
 class IProduct(Interface):
-    export_as_webservice_entry()
     title = exported(TextLine(title=u'The product title'))
     # Need to define the three attributes below because we have a test which
     # wraps a Product object with a security proxy and later uses adapters
@@ -344,8 +558,8 @@ class IProduct(Interface):
     _bug_target_name = Attribute('bug_target_name')
 
 
+@implementer(IProduct)
 class Product(object):
-    implements(IProduct)
     title = 'A product'
     _bug_count = 0
     _dev_branch = None
@@ -353,19 +567,19 @@ class Product(object):
     _bug_target_name = None
 
 
+@exported_as_webservice_entry()
 class IProject(Interface):
-    export_as_webservice_entry()
     title = exported(TextLine(title=u'The project title'))
 
 
+@implementer(IProject)
 class Project(object):
-    implements(IProject)
     title = 'A project'
     _bug_count = 0
 
 
+@exported_as_webservice_entry(contributes_to=[IProduct, IProject])
 class IHasBugs(Interface):
-    export_as_webservice_entry(contributes_to=[IProduct, IProject])
     bug_count = exported(Int(title=u'Number of bugs'))
     not_exported = TextLine(title=u'Not exported')
     bug_target_name = exported(
@@ -401,14 +615,14 @@ class IHasBugs(Interface):
         pass
 
 
+@exported_as_webservice_entry(contributes_to=[IProduct])
 class IHasBugs2(Interface):
-    export_as_webservice_entry(contributes_to=[IProduct])
     bug_count = exported(Int(title=u'Number of bugs'))
     not_exported = TextLine(title=u'Not exported')
 
 
+@exported_as_webservice_entry(contributes_to=[IProduct])
 class IHasBugs3(Interface):
-    export_as_webservice_entry(contributes_to=[IProduct])
     not_exported = TextLine(title=u'Not exported')
 
     @export_read_operation()
@@ -416,9 +630,9 @@ class IHasBugs3(Interface):
         pass
 
 
+@adapter(IProduct)
+@implementer(IHasBugs)
 class ProductToHasBugsAdapter(object):
-    adapts(IProduct)
-    implements(IHasBugs)
 
     def __init__(self, context):
         self.context = context
@@ -437,24 +651,25 @@ class ProductToHasBugsAdapter(object):
         self.context._bug_target_name = value
 
 
+@adapter(IProject)
 class ProjectToHasBugsAdapter(ProductToHasBugsAdapter):
-    adapts(IProject)
+    pass
 
 
+@exported_as_webservice_entry()
 class IBranch(Interface):
-    export_as_webservice_entry()
     name = TextLine(title=u'The branch name')
 
 
+@implementer(IBranch)
 class Branch(object):
-    implements(IBranch)
 
     def __init__(self, name):
         self.name = name
 
 
+@exported_as_webservice_entry(contributes_to=[IProduct])
 class IHasBranches(Interface):
-    export_as_webservice_entry(contributes_to=[IProduct])
     not_exported = TextLine(title=u'Not exported')
     development_branch = exported(
         Reference(schema=IBranch, readonly=True),
@@ -479,9 +694,9 @@ class IHasBranches(Interface):
         pass
 
 
+@adapter(IProduct)
+@implementer(IHasBranches)
 class ProductToHasBranchesAdapter(object):
-    adapts(IProduct)
-    implements(IHasBranches)
 
     def __init__(self, context):
         self.context = context
@@ -501,37 +716,43 @@ class ProductToHasBranchesAdapter(object):
 # don't care about the unmarshalling itself, we need to register a generic
 # marshaller so that the adapter lookup doesn't fail and cause an error on the
 # test.
+@adapter(Interface, IHTTPRequest)
 class DummyFieldMarshaller(SimpleFieldMarshaller):
-    adapts(Interface, IHTTPRequest)
+    pass
+
 
 # Classes for TestEntryMultiversion.
 
+@exported_as_webservice_entry(as_of="2.0")
 class INotInitiallyExported(Interface):
     # An entry that's not exported in the first version of the web
     # service.
-    export_as_webservice_entry(as_of="2.0")
+    pass
 
 
+@exported_as_webservice_entry(
+    versioned_annotations=[('2.0', dict(exported=False))])
 class INotPresentInLaterVersion(Interface):
     # An entry that's only exported in the first version of the web
     # service.
-    export_as_webservice_entry(
-        versioned_annotations=[('2.0', dict(exported=False))])
+    pass
 
 
+@exported_as_webservice_entry(
+    singular_name="octopus", plural_name="octopi",
+    versioned_annotations=[
+        ('2.0', dict(singular_name="fish", plural_name="fishes"))])
 class IHasDifferentNamesInDifferentVersions(Interface):
     # An entry that has different names in different versions.
-    export_as_webservice_entry(
-        singular_name="octopus", plural_name="octopi",
-        versioned_annotations=[
-            ('2.0', dict(singular_name="fish", plural_name="fishes"))])
+    pass
 
 
+@exported_as_webservice_entry(
+    singular_name="frog",
+    versioned_annotations=[('2.0', dict(singular_name="toad"))])
 class IHasDifferentSingularNamesInDifferentVersions(Interface):
     # An entry that has different names in different versions.
-    export_as_webservice_entry(
-        singular_name="frog",
-        versioned_annotations=[('2.0', dict(singular_name="toad"))])
+    pass
 
 
 class TestEntryMultiversion(TestCaseWithWebServiceFixtures):
@@ -542,8 +763,8 @@ class TestEntryMultiversion(TestCaseWithWebServiceFixtures):
         interfaces = generate_entry_interfaces(
             INotInitiallyExported, [],
             *getUtility(IWebServiceConfiguration).active_versions)
-        self.assertEquals(len(interfaces), 1)
-        self.assertEquals(interfaces[0].version, '2.0')
+        self.assertEqual(len(interfaces), 1)
+        self.assertEqual(interfaces[0].version, '2.0')
 
     def test_not_exported_in_later_version(self):
         # INotPresentInLaterVersion is published in version 1.0 but
@@ -551,9 +772,9 @@ class TestEntryMultiversion(TestCaseWithWebServiceFixtures):
         interfaces = generate_entry_interfaces(
             INotPresentInLaterVersion, [],
             *getUtility(IWebServiceConfiguration).active_versions)
-        self.assertEquals(len(interfaces), 1)
+        self.assertEqual(len(interfaces), 1)
         tags = interfaces[0][1].getTaggedValue(LAZR_WEBSERVICE_NAME)
-        self.assertEquals(interfaces[0].version, '1.0')
+        self.assertEqual(interfaces[0].version, '1.0')
 
     def test_different_names_in_different_versions(self):
         # IHasDifferentNamesInDifferentVersions is called
@@ -563,25 +784,26 @@ class TestEntryMultiversion(TestCaseWithWebServiceFixtures):
             *getUtility(IWebServiceConfiguration).active_versions)
         interface_10 = interfaces[0].object
         tags_10 = interface_10.getTaggedValue(LAZR_WEBSERVICE_NAME)
-        self.assertEquals('octopus', tags_10['singular'])
-        self.assertEquals('octopi', tags_10['plural'])
+        self.assertEqual('octopus', tags_10['singular'])
+        self.assertEqual('octopi', tags_10['plural'])
 
         interface_20 = interfaces[1].object
         tags_20 = interface_20.getTaggedValue(LAZR_WEBSERVICE_NAME)
-        self.assertEquals('fish', tags_20['singular'])
-        self.assertEquals('fishes', tags_20['plural'])
+        self.assertEqual('fish', tags_20['singular'])
+        self.assertEqual('fishes', tags_20['plural'])
 
     def test_nonexistent_annotation_fails(self):
         # You can't define an entry class that includes an unrecognized
         # annotation in its versioned_annotations.
         try:
+            @exported_as_webservice_entry(
+                versioned_annotations=[
+                    ('2.0', dict(no_such_annotation=True))])
             class IUsesNonexistentAnnotation(Interface):
-                export_as_webservice_entry(
-                    versioned_annotations=[
-                        ('2.0', dict(no_such_annotation=True))])
+                pass
             self.fail("Expected ValueError.")
-        except ValueError, exception:
-            self.assertEquals(
+        except ValueError as exception:
+            self.assertEqual(
                 str(exception),
                 'Unrecognized annotation for version "2.0": '
                 '"no_such_annotation"')
@@ -595,69 +817,66 @@ class TestEntryMultiversion(TestCaseWithWebServiceFixtures):
             *getUtility(IWebServiceConfiguration).active_versions)
         interface_10 = interfaces[0].object
         tags_10 = interface_10.getTaggedValue(LAZR_WEBSERVICE_NAME)
-        self.assertEquals('frog', tags_10['singular'])
-        self.assertEquals('frogs', tags_10['plural'])
+        self.assertEqual('frog', tags_10['singular'])
+        self.assertEqual('frogs', tags_10['plural'])
 
         interface_20 = interfaces[1].object
         tags_20 = interface_20.getTaggedValue(LAZR_WEBSERVICE_NAME)
-        self.assertEquals('toad', tags_20['singular'])
-        self.assertEquals('toads', tags_20['plural'])
+        self.assertEqual('toad', tags_20['singular'])
+        self.assertEqual('toads', tags_20['plural'])
 
 
 # Classes for TestReqireExplicitVersions
 
+@exported_as_webservice_entry()
 class IEntryExportedWithoutAsOf(Interface):
-    export_as_webservice_entry()
+    pass
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IEntryExportedWithAsOf(Interface):
-    export_as_webservice_entry(as_of="1.0")
+    pass
 
 
+@exported_as_webservice_entry(as_of="2.0")
 class IEntryExportedAsOfLaterVersion(Interface):
-    export_as_webservice_entry(as_of="2.0")
+    pass
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldExportedWithoutAsOf(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     field = exported(TextLine(), exported=True)
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldExportedToEarliestVersionUsingAsOf(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     field = exported(TextLine(), as_of='1.0')
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldExportedToLatestVersionUsingAsOf(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     field = exported(TextLine(), as_of='2.0')
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldDefiningAttributesBeforeAsOf(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     field = exported(TextLine(), ('1.0', dict(exported=True)),
                      as_of='2.0')
 
-class IFieldAsOfNonexistentVersion(Interface):
-    export_as_webservice_entry(as_of="1.0")
 
+@exported_as_webservice_entry(as_of="1.0")
+class IFieldAsOfNonexistentVersion(Interface):
     field = exported(TextLine(), as_of='nosuchversion')
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldDoubleDefinition(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     field = exported(TextLine(), ('2.0', dict(exported_as='name2')),
                      exported_as='name2', as_of='2.0')
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldImplicitOperationDefinition(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     @call_with(value="2.0")
     @operation_for_version('2.0')
     @call_with(value="1.0")
@@ -666,9 +885,8 @@ class IFieldImplicitOperationDefinition(Interface):
         pass
 
 
+@exported_as_webservice_entry(as_of="1.0")
 class IFieldExplicitOperationDefinition(Interface):
-    export_as_webservice_entry(as_of="1.0")
-
     @call_with(value="2.0")
     @operation_for_version('2.0')
     @call_with(value="1.0")
@@ -678,8 +896,9 @@ class IFieldExplicitOperationDefinition(Interface):
         pass
 
 
+@implementer(IFieldExplicitOperationDefinition)
 class FieldExplicitOperationDefinition(object):
-    implements(IFieldExplicitOperationDefinition)
+    pass
 
 
 class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
@@ -696,9 +915,9 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         interfaces = generate_entry_interfaces(
             IEntryExportedWithAsOf, [],
             *self.utility.active_versions)
-        self.assertEquals(len(interfaces), 2)
-        self.assertEquals(interfaces[0].version, '1.0')
-        self.assertEquals(interfaces[1].version, '2.0')
+        self.assertEqual(len(interfaces), 2)
+        self.assertEqual(interfaces[0].version, '1.0')
+        self.assertEqual(interfaces[1].version, '2.0')
 
     def test_entry_exported_as_of_later_version_succeeds(self):
         # An entry exported as_of a later version is not present in
@@ -706,15 +925,15 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         interfaces = generate_entry_interfaces(
             IEntryExportedAsOfLaterVersion, [],
             *self.utility.active_versions)
-        self.assertEquals(len(interfaces), 1)
-        self.assertEquals(interfaces[0].version, '2.0')
+        self.assertEqual(len(interfaces), 1)
+        self.assertEqual(interfaces[0].version, '2.0')
 
     def test_entry_exported_without_as_of_fails(self):
         exception = self.assertRaises(
             ValueError, generate_entry_interfaces,
             IEntryExportedWithoutAsOf, [],
             *self.utility.active_versions)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             'Entry "IEntryExportedWithoutAsOf": Exported in version 1.0, '
             'but not by using as_of. The service configuration requires '
@@ -729,8 +948,8 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
             *self.utility.active_versions)
         interface_10 = interfaces[0].object
         interface_20 = interfaces[1].object
-        self.assertEquals(interface_10.names(), ['field'])
-        self.assertEquals(interface_20.names(), ['field'])
+        self.assertEqual(interface_10.names(), ['field'])
+        self.assertEqual(interface_20.names(), ['field'])
 
     def test_field_exported_as_of_later_version_is_not_exported_in_earlier_versions(self):
         # If you export a field as_of version 2.0, it's not present in
@@ -740,8 +959,8 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
             *self.utility.active_versions)
         interface_10 = interfaces[0].object
         interface_20 = interfaces[1].object
-        self.assertEquals(interface_10.names(), [])
-        self.assertEquals(interface_20.names(), ['field'])
+        self.assertEqual(interface_10.names(), [])
+        self.assertEqual(interface_20.names(), ['field'])
 
     def test_field_not_exported_using_as_of_fails(self):
         # If you export a field without specifying as_of, you get an
@@ -749,7 +968,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         exception = self.assertRaises(
             ValueError, generate_entry_interfaces,
             IFieldExportedWithoutAsOf, [], *self.utility.active_versions)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             ('Field "field" in interface "IFieldExportedWithoutAsOf": '
              'Exported in version 1.0, but not by using as_of. '
@@ -761,7 +980,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         # the exported keyword argument to False.
         exception = self.assertRaises(
             ValueError, exported, TextLine(), as_of='1.0', exported=False)
-        self.assertEquals(
+        self.assertEqual(
                 str(exception),
                 ('as_of=1.0 says to export TextLine, but exported=False '
                  'says not to.'))
@@ -772,7 +991,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
             ValueError, generate_entry_interfaces,
             IFieldAsOfNonexistentVersion, [],
             *self.utility.active_versions)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             ('Field "field" in interface "IFieldAsOfNonexistentVersion": '
              'Unrecognized version "nosuchversion".'))
@@ -783,7 +1002,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         exception = self.assertRaises(
             ValueError, generate_entry_interfaces,
             IFieldDoubleDefinition, [], *self.utility.active_versions)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             ('Field "field" in interface "IFieldDoubleDefinition": '
              'Duplicate definitions for version "2.0".'))
@@ -795,7 +1014,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
             ValueError, generate_entry_interfaces,
             IFieldDefiningAttributesBeforeAsOf, [],
             *self.utility.active_versions)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             ('Field "field" in interface '
              '"IFieldDefiningAttributesBeforeAsOf": Version "1.0" defined '
@@ -812,7 +1031,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
                   if name == 'implicitly_in_10'][0]
         exception = self.assertRaises(
             ValueError, generate_operation_adapter, method, None)
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             '"implicitly_in_10" is implicitly tagged for export to web '
             'service version "1.0", but the service configuration requires '
@@ -825,7 +1044,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         exception = self.assertRaises(
             ValueError, generate_and_register_webservice_operations,
             None, IFieldImplicitOperationDefinition, [])
-        self.assertEquals(
+        self.assertEqual(
             str(exception),
             ('IFieldImplicitOperationDefinition.implicitly_in_10: '
              'Implicitly tagged for export to web service version "1.0", '
@@ -845,7 +1064,7 @@ class TestRequireExplicitVersions(TestCaseWithWebServiceFixtures):
         adapter = getMultiAdapter(
             (context, self.fake_request('1.0')),
             IResourceGETOperation, name='explicitly_in_10')
-        self.assertEquals(
+        self.assertEqual(
             adapter.__class__.__name__,
             'GET_IFieldExplicitOperationDefinition_explicitly_in_10_1_0')
 
@@ -856,47 +1075,43 @@ class INotPublished(Interface):
     pass
 
 
+@exported_as_webservice_entry()
 class IReferencesNotPublished(Interface):
-    export_as_webservice_entry()
     field = exported(Reference(schema=INotPublished))
 
 
+@exported_as_webservice_entry()
 class IReferencesCollectionOfNotPublished(Interface):
-    export_as_webservice_entry()
     field = exported(
         CollectionField(value_type=Reference(schema=INotPublished)))
 
 
+@exported_as_webservice_entry()
 class IOperationReturnsNotPublished(Interface):
-    export_as_webservice_entry()
-
     @operation_returns_entry(INotPublished)
     @export_read_operation()
     def get_impossible_object():
         pass
 
 
+@exported_as_webservice_entry()
 class IOperationReturnsNotPublishedCollection(Interface):
-    export_as_webservice_entry()
-
     @operation_returns_collection_of(INotPublished)
     @export_read_operation()
     def get_impossible_objects():
         pass
 
 
+@exported_as_webservice_entry()
 class IOperationAcceptsNotPublished(Interface):
-    export_as_webservice_entry()
-
     @operation_parameters(arg=Reference(schema=INotPublished))
     @export_write_operation()
     def use_impossible_object(arg):
         pass
 
 
+@exported_as_webservice_entry()
 class IOperationAcceptsCollectionOfNotPublished(Interface):
-    export_as_webservice_entry()
-
     @operation_parameters(
         arg=CollectionField(value_type=Reference(schema=INotPublished)))
     @export_write_operation()
@@ -904,31 +1119,35 @@ class IOperationAcceptsCollectionOfNotPublished(Interface):
         pass
 
 
+@exported_as_webservice_entry(as_of='2.0')
 class IPublishedTooLate(Interface):
-    export_as_webservice_entry(as_of='2.0')
+    pass
 
 
+@exported_as_webservice_entry(as_of='1.0')
 class IReferencesPublishedTooLate(Interface):
-    export_as_webservice_entry(as_of='1.0')
     field = exported(Reference(schema=IPublishedTooLate))
 
 
+@exported_as_webservice_entry(as_of='1.0')
 class IPublishedEarly(Interface):
-    export_as_webservice_entry(as_of='1.0')
+    pass
 
 
+@exported_as_webservice_entry(as_of='2.0')
 class IPublishedLate(Interface):
-    export_as_webservice_entry(as_of='2.0')
     field = exported(Reference(schema=IPublishedEarly))
 
 
+@exported_as_webservice_entry(
+    as_of='1.0', versioned_annotations=[
+        VersionedObject('2.0', dict(exported=False))])
 class IPublishedAndThenRemoved(Interface):
-    export_as_webservice_entry(
-        as_of='1.0', versioned_annotations=[
-            VersionedObject('2.0', dict(exported=False))])
+    pass
 
+
+@exported_as_webservice_entry(as_of='1.0')
 class IReferencesPublishedAndThenRemoved(Interface):
-    export_as_webservice_entry(as_of='1.0')
     field = exported(Reference(schema=IPublishedAndThenRemoved))
 
 
@@ -966,7 +1185,7 @@ class TestSanityChecking(TestCaseWithWebServiceFixtures):
                 version=expect_failure_in_version,
                 reason=expect_failure_for_reason,
                 interface=expect_failure_due_to_interface.__name__))
-        self.assertEquals(str(exception), expected_message)
+        self.assertEqual(str(exception), expected_message)
 
 
     def test_reference_to_unpublished_object_fails(self):
@@ -999,8 +1218,7 @@ class TestSanityChecking(TestCaseWithWebServiceFixtures):
         # object is also present in 2.0.
 
         # We'll call this test a success if it doesn't raise an exception.
-        module = register_test_module(
-            'testmod', IPublishedEarly, IPublishedLate)
+        register_test_module('testmod', IPublishedEarly, IPublishedLate)
 
     # At this point we've tested all the ways entries might not be
     # published in a given version: they might never be published,
