@@ -2,12 +2,22 @@
 
 """Tests of lazr.restful navigation."""
 
+from __future__ import absolute_import, print_function
+
 __metaclass__ = type
 
 import unittest
 
-from zope.component import getSiteManager
-from zope.interface import Interface, implements
+from zope.component import (
+    ComponentLookupError,
+    getMultiAdapter,
+    getSiteManager,
+    )
+from zope.interface import (
+    alsoProvides,
+    Interface,
+    implementer,
+    )
 from zope.publisher.interfaces import NotFound
 from zope.schema import Text
 from zope.testing.cleanup import cleanUp
@@ -33,9 +43,9 @@ class IParent(Interface):
     child = Reference(schema=IChild)
 
 
+@implementer(IChild)
 class Child:
     """A simple implementation of IChild."""
-    implements(IChild)
     one = u'one'
     two = u'two'
 
@@ -43,13 +53,14 @@ class Child:
 class ChildEntry:
     """Implementation of an entry wrapping a Child."""
     schema = IChild
+
     def __init__(self, context, request):
         self.context = context
 
 
+@implementer(IParent)
 class Parent:
     """A simple implementation of IParent."""
-    implements(IParent)
     three = u'three'
     child = Child()
 
@@ -106,6 +117,24 @@ class NavigationTestCase(unittest.TestCase):
             NotFound, publication.traverseName,
             request, parent, 'child')
 
+    def test_getResource_ComponentLookupError_NotFound(self):
+        # Test that ComponentLookupError becomes NotFound.
+        # IEntry objects may not be adaptable to a resource if
+        # the underlying object cannot be found.
+        class IOrphan(Interface):
+            pass
 
-def additional_tests():
-    return unittest.TestLoader().loadTestsFromName(__name__)
+        class OrphanEntry:
+            schema = IOrphan
+
+            def __init__(self):
+                alsoProvides(self, IEntry)
+
+        publication = Publication(None)
+        request = FakeRequestWithEmptyTraversalStack(version='trunk')
+        entry_obj = OrphanEntry()
+        self.assertRaises(
+            ComponentLookupError,
+            getMultiAdapter, (entry_obj, request), IEntry)
+        self.assertRaises(
+            NotFound, publication.getResource, request, entry_obj)
